//--------------------------------------------------------------
// File     : main.c
// Datum    : 17.06.2015
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// Board    : STM32F429-Board von UB (Version 1.0)
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : CMSIS_BOOT, M4_CMSIS_CORE
// Funktion : Hauptprogramm
// Hinweis  : Diese zwei Files muessen auf 8MHz stehen
//              "cmsis_boot/stm32f4xx.h"
//              "cmsis_boot/system_stm32f4xx.c"
// In Configuration diese Define hinzufgen :
// "STM32F429_439xx" , "__ASSEMBLY__" , "USE_STDPERIPH_DRIVER"
//--------------------------------------------------------------
// Beschreibung :
// LoLevel-Test vom eigenbau STM32F429-Board von UB (Version 1.0)
//
// Check der TFT GPIOs auf Kurzschlusse
// (diese Software nur benutzen, wenn kein TFT am TFT-Adapter angeschlossen ist)
//
// Statusausgabe per COM-6 (3,3V UART) 115200 Baud
//--------------------------------------------------------------

#include "main.h"
#include "stm32_ub_systick.h"
#include "stm32_ub_uart.h"
#include <stdio.h>


//--------------------------------------------------------------
// defines welche Pins geprft werden sollen (nicht aendern !!)
// Bit : 1 => Pin wird geprft
// Bit : 0 => Pin wird nicht geprft
//--------------------------------------------------------------
#define  GPIOA_PINS  0x000F // PA0-3
#define  GPIOB_PINS  0x0000 // keine
#define  GPIOC_PINS  0x0000 // keine
#define  GPIOD_PINS  0x0000 // keine
#define  GPIOE_PINS  0x0070 // PE4-6
#define  GPIOF_PINS  0x0400 // PF10
#define  GPIOG_PINS  0x1CC0 // PG6-7,10-12
#define  GPIOH_PINS  0xFF0C // PH2-3,8-15
#define  GPIOI_PINS  0x06F7 // PI0-2,4-7,9-10


//----------------------------------------------------------------------
// alle GPIOS auf Eingang mit PullUp schalten
//----------------------------------------------------------------------
void init_ports_pullup(void)
{
  GPIO_InitTypeDef  GPIO_InitStructure;

  UB_Uart_SendString(COM6,"-----------------------------------",CRLF);
  UB_Uart_SendString(COM6,"set all GPIOs to input with pull-up",CRLF);
  UB_Uart_SendString(COM6,"-----------------------------------",CRLF);

  // Clocks
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOF, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOG, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOH, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOI, ENABLE);

  // set pin as input (PullUp)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOA_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  // set pin as input (PullUp)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOB_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  // set pin as input (PullUp)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOC_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  // set pin as input (PullUp)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOD_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  // set pin as input (PullUp)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOE_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOE, &GPIO_InitStructure);

  // set pin as input (PullUp)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOF_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOF, &GPIO_InitStructure);

  // set pin as input (PullUp)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOG_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOG, &GPIO_InitStructure);

  // set pin as input (PullUp)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOH_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOH, &GPIO_InitStructure);

  // set pin as input (PullUp)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOI_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(GPIOI, &GPIO_InitStructure);
}

//----------------------------------------------------------------------
// alle GPIOS auf Eingang mit PullDown schalten
//----------------------------------------------------------------------
void init_ports_pulldown(void)
{
  GPIO_InitTypeDef  GPIO_InitStructure;

  UB_Uart_SendString(COM6,"-------------------------------------",CRLF);
  UB_Uart_SendString(COM6,"set all GPIOs to input with pull-down",CRLF);
  UB_Uart_SendString(COM6,"-------------------------------------",CRLF);

  // Clocks
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOF, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOG, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOH, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOI, ENABLE);

  // set pin as input (PullDown)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOA_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  // set pin as input (PullDown)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOB_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  // set pin as input (PullDown)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOC_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  // set pin as input (PullDown)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOD_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  // set pin as input (PullDown)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOE_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
  GPIO_Init(GPIOE, &GPIO_InitStructure);

  // set pin as input (PullDown)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOF_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
  GPIO_Init(GPIOF, &GPIO_InitStructure);

  // set pin as input (PullDown)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOG_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
  GPIO_Init(GPIOG, &GPIO_InitStructure);

  // set pin as input (PullDown)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOH_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
  GPIO_Init(GPIOH, &GPIO_InitStructure);

  // set pin as input (PullDown)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIOI_PINS;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
  GPIO_Init(GPIOI, &GPIO_InitStructure);
}


//----------------------------------------------------------------------
// check ob alle Pins der Maske ein Hi-Signal liefern
// falls nicht -> fehler
//----------------------------------------------------------------------
uint8_t check_gpio_hi(GPIO_TypeDef* port, uint32_t maske)
{
  uint8_t check=1;

  if((maske&0x0001)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_0);
  if(check==0) return(1);
  if((maske&0x0002)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_1);
  if(check==0) return(2);
  if((maske&0x0004)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_2);
  if(check==0) return(3);
  if((maske&0x0008)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_3);
  if(check==0) return(4);
  if((maske&0x0010)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_4);
  if(check==0) return(5);
  if((maske&0x0020)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_5);
  if(check==0) return(6);
  if((maske&0x0040)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_6);
  if(check==0) return(7);
  if((maske&0x0080)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_7);
  if(check==0) return(8);
  if((maske&0x0100)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_8);
  if(check==0) return(9);
  if((maske&0x0200)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_9);
  if(check==0) return(10);
  if((maske&0x0400)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_10);
  if(check==0) return(11);
  if((maske&0x0800)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_11);
  if(check==0) return(12);
  if((maske&0x1000)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_12);
  if(check==0) return(13);
  if((maske&0x2000)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_13);
  if(check==0) return(14);
  if((maske&0x4000)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_14);
  if(check==0) return(15);
  if((maske&0x8000)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_15);
  if(check==0) return(16);

  return(0);
}

//----------------------------------------------------------------------
// check ob ein Pin mit GND kurzgeschlossen ist
//
// Ablauf : 
//   1. Alle GPIOs auf Eingang mit PullUp schalten
//   2. Check ob alle Pins ein Hi-Pegel fuehren
//   3. Falls ein Pin Lo-Pegel hat -> fehler
//----------------------------------------------------------------------
uint8_t check_ports_hi(void)
{
  uint8_t ret_wert=0;
  char buf[20];
  uint8_t check=0;

  UB_Uart_SendString(COM6,"=====================",CRLF);
  UB_Uart_SendString(COM6,"check HI at all Ports",CRLF);
  UB_Uart_SendString(COM6,"=====================",CRLF);
  UB_Uart_SendString(COM6," ",CRLF);

  UB_Systick_Pause_ms(500);
  // Port-A
  UB_Uart_SendString(COM6,"check Hi at Port-A[0-3]",CRLF);
  check=check_gpio_hi(GPIOA, GPIOA_PINS);
  if(check==0) {
    UB_Uart_SendString(COM6,"Port-A ok",CRLF);
  }
  else {
    UB_Uart_SendString(COM6,"Port-A error !!",CRLF);
    sprintf(buf,"first error at PA[%d]",check-1);
    UB_Uart_SendString(COM6,buf,CRLF);
    ret_wert=1;
  }
  // Port-B
  UB_Uart_SendString(COM6,"skip Port-B",CRLF);
  // Port-C
  UB_Uart_SendString(COM6,"skip Port-C",CRLF);
  // Port-D
  UB_Uart_SendString(COM6,"skip Port-D",CRLF);
  // Port-E
  UB_Uart_SendString(COM6,"check Hi at Port-E[4-6]",CRLF);
  check=check_gpio_hi(GPIOE, GPIOE_PINS);
  if(check==0) {
    UB_Uart_SendString(COM6,"Port-E ok",CRLF);
  }
  else {
    UB_Uart_SendString(COM6,"Port-E error !!",CRLF);
    sprintf(buf,"first error at PE[%d]",check-1);
    UB_Uart_SendString(COM6,buf,CRLF);
    ret_wert=1;
  }
  // Port-F
  UB_Uart_SendString(COM6,"check Hi at Port-F[10]",CRLF);
  check=check_gpio_hi(GPIOF, GPIOF_PINS);
  if(check==0) {
    UB_Uart_SendString(COM6,"Port-F ok",CRLF);
  }
  else {
    UB_Uart_SendString(COM6,"Port-F error !!",CRLF);
    sprintf(buf,"first error at PF[%d]",check-1);
    UB_Uart_SendString(COM6,buf,CRLF);
    ret_wert=1;
  }
  // Port-G
  UB_Uart_SendString(COM6,"check Hi at Port-G[6-7,10-12]",CRLF);
  check=check_gpio_hi(GPIOG, GPIOG_PINS);
  if(check==0) {
    UB_Uart_SendString(COM6,"Port-G ok",CRLF);
  }
  else {
    UB_Uart_SendString(COM6,"Port-G error !!",CRLF);
    sprintf(buf,"first error at PG[%d]",check-1);
    UB_Uart_SendString(COM6,buf,CRLF);
    ret_wert=1;
  }
  // Port-H
  UB_Uart_SendString(COM6,"check Hi at Port-H[2-3,8-15]",CRLF);
  check=check_gpio_hi(GPIOH, GPIOH_PINS);
  if(check==0) {
    UB_Uart_SendString(COM6,"Port-H ok",CRLF);
  }
  else {
    UB_Uart_SendString(COM6,"Port-H error !!",CRLF);
    sprintf(buf,"first error at PH[%d]",check-1);
    UB_Uart_SendString(COM6,buf,CRLF);
    ret_wert=1;
  }
  // Port-I
  UB_Uart_SendString(COM6,"check Hi at Port-I[0-2,4-7,9-10]",CRLF);
  check=check_gpio_hi(GPIOI, GPIOI_PINS);
  if(check==0) {
    UB_Uart_SendString(COM6,"Port-I ok",CRLF);
  }
  else {
    UB_Uart_SendString(COM6,"Port-I error !!",CRLF);
    sprintf(buf,"first error at PI[%d]",check-1);
    UB_Uart_SendString(COM6,buf,CRLF);
    ret_wert=1;
  }

  return(ret_wert);
}


//----------------------------------------------------------------------
// check ob alle Pins der Maske ein Lo-Signal liefern
// falls nicht -> fehler
//----------------------------------------------------------------------
uint8_t check_gpio_lo(GPIO_TypeDef* port, uint32_t maske)
{
  uint8_t check=0;

  if((maske&0x0001)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_0);
  if(check!=0) return(1);
  if((maske&0x0002)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_1);
  if(check!=0) return(2);
  if((maske&0x0004)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_2);
  if(check!=0) return(3);
  if((maske&0x0008)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_3);
  if(check!=0) return(4);
  if((maske&0x0010)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_4);
  if(check!=0) return(5);
  if((maske&0x0020)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_5);
  if(check!=0) return(6);
  if((maske&0x0040)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_6);
  if(check!=0) return(7);
  if((maske&0x0080)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_7);
  if(check!=0) return(8);
  if((maske&0x0100)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_8);
  if(check!=0) return(9);
  if((maske&0x0200)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_9);
  if(check!=0) return(10);
  if((maske&0x0400)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_10);
  if(check!=0) return(11);
  if((maske&0x0800)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_11);
  if(check!=0) return(12);
  if((maske&0x1000)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_12);
  if(check!=0) return(13);
  if((maske&0x2000)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_13);
  if(check!=0) return(14);
  if((maske&0x4000)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_14);
  if(check!=0) return(15);
  if((maske&0x8000)!=0) check=GPIO_ReadInputDataBit(port, GPIO_Pin_15);
  if(check!=0) return(16);

  return(0);
}

//----------------------------------------------------------------------
// check ob ein Pin mit VCC kurzgeschlossen ist
//
// Ablauf : 
//   1. Alle GPIOs auf Eingang mit PullDown schalten
//   2. Check ob alle Pins ein Lo-Pegel fuehren
//   3. Falls ein Pin Hi-Pegel hat -> fehler
//----------------------------------------------------------------------
uint8_t check_ports_lo(void)
{
  uint8_t ret_wert=0;
  char buf[20];
  uint8_t check=0;

  UB_Uart_SendString(COM6,"=====================",CRLF);
  UB_Uart_SendString(COM6,"check LO at all Ports",CRLF);
  UB_Uart_SendString(COM6,"=====================",CRLF);
  UB_Uart_SendString(COM6," ",CRLF);

  UB_Systick_Pause_ms(500);
  // Port-A
  UB_Uart_SendString(COM6,"check Lo at Port-A[0-3]",CRLF);
  check=check_gpio_lo(GPIOA, GPIOA_PINS);
  if(check==0) {
    UB_Uart_SendString(COM6,"Port-A ok",CRLF);
  }
  else {
    UB_Uart_SendString(COM6,"Port-A error !!",CRLF);
    sprintf(buf,"first error at PA[%d]",check-1);
    UB_Uart_SendString(COM6,buf,CRLF);
    ret_wert=1;
  }
  // Port-B
  UB_Uart_SendString(COM6,"skip Port-B",CRLF);
  // Port-C
  UB_Uart_SendString(COM6,"skip Port-C",CRLF);
  // Port-D
  UB_Uart_SendString(COM6,"skip Port-D",CRLF);
  // Port-E
  UB_Uart_SendString(COM6,"check Lo at Port-E[4-6]",CRLF);
  check=check_gpio_lo(GPIOE, GPIOE_PINS);
  if(check==0) {
    UB_Uart_SendString(COM6,"Port-E ok",CRLF);
  }
  else {
    UB_Uart_SendString(COM6,"Port-E error !!",CRLF);
    sprintf(buf,"first error at PE[%d]",check-1);
    UB_Uart_SendString(COM6,buf,CRLF);
    ret_wert=1;
  }
  // Port-F
  UB_Uart_SendString(COM6,"check Lo at Port-F[10]",CRLF);
  check=check_gpio_lo(GPIOF, GPIOF_PINS);
  if(check==0) {
    UB_Uart_SendString(COM6,"Port-F ok",CRLF);
  }
  else {
    UB_Uart_SendString(COM6,"Port-F error !!",CRLF);
    sprintf(buf,"first error at PF[%d]",check-1);
    UB_Uart_SendString(COM6,buf,CRLF);
    ret_wert=1;
  }
  // Port-G
  UB_Uart_SendString(COM6,"check Lo at Port-G[6-7,10-12]",CRLF);
  check=check_gpio_lo(GPIOG, GPIOG_PINS);
  if(check==0) {
    UB_Uart_SendString(COM6,"Port-G ok",CRLF);
  }
  else {
    UB_Uart_SendString(COM6,"Port-G error !!",CRLF);
    sprintf(buf,"first error at PG[%d]",check-1);
    UB_Uart_SendString(COM6,buf,CRLF);
    ret_wert=1;
  }
  // Port-H
  UB_Uart_SendString(COM6,"check Lo at Port-H[2-3,8-15]",CRLF);
  check=check_gpio_lo(GPIOH, GPIOH_PINS);
  if(check==0) {
    UB_Uart_SendString(COM6,"Port-H ok",CRLF);
  }
  else {
    UB_Uart_SendString(COM6,"Port-H error !!",CRLF);
    sprintf(buf,"first error at PH[%d]",check-1);
    UB_Uart_SendString(COM6,buf,CRLF);
    ret_wert=1;
  }
  // Port-I
  UB_Uart_SendString(COM6,"check Lo at Port-I[0-2,4-7,9-10]",CRLF);
  check=check_gpio_lo(GPIOI, GPIOI_PINS);
  if(check==0) {
    UB_Uart_SendString(COM6,"Port-I ok",CRLF);
  }
  else {
    UB_Uart_SendString(COM6,"Port-I error !!",CRLF);
    sprintf(buf,"first error at PI[%d]",check-1);
    UB_Uart_SendString(COM6,buf,CRLF);
    ret_wert=1;
  }

  return(ret_wert);
}

//----------------------------------------------------------------------
// einen Pin eines Ports auf Ausgang schalten (Pegel = Lo)
//----------------------------------------------------------------------
void set_gpio_out(GPIO_TypeDef* port, uint32_t pin)
{
  GPIO_InitTypeDef  GPIO_InitStructure;

  // set pin as output (pushpull)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = pin;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(port, &GPIO_InitStructure);
  UB_Systick_Pause_ms(5);
  port->BSRRH = pin;  // pin auf Lo
  UB_Systick_Pause_ms(5);
}

//----------------------------------------------------------------------
// einen Pin eines Ports auf Eingang schalten (mit PullUp)
//----------------------------------------------------------------------
void set_gpio_in(GPIO_TypeDef* port, uint32_t pin)
{
  GPIO_InitTypeDef  GPIO_InitStructure;

  port->BSRRL = pin; // pin auf Hi
  UB_Systick_Pause_ms(5);
  // set pin as input (PullUp)
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = pin;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_Init(port, &GPIO_InitStructure);
  UB_Systick_Pause_ms(5);
}

//----------------------------------------------------------------------
// check ob ein Pin mit einem anderen Pin kurzgeschlossen ist
//
// Ablauf : 
//   1. Alle GPIOs stehen auf Eingang mit PullUp (alle fhren also Hi-Pegel)
//   2. Jeden Pin einzeln auf Ausgang mit Lo-Pegel schalten
//   3. Danach alle anderen Pins prfen ob irgendeiner Lo-Pegel fhrt
//   4. Falls ein Pin Lo-Pegel hat -> fehler
//----------------------------------------------------------------------
uint8_t check_all_gpios(void)
{
  uint8_t c1,c2,c3,c4,c5,c6=0;
  char buf[20];
  uint32_t soll_maske;
  uint32_t pin_maske=0x0001;
  uint8_t n;

  UB_Uart_SendString(COM6,"==================",CRLF);
  UB_Uart_SendString(COM6,"check GPIO to GPIO",CRLF);
  UB_Uart_SendString(COM6,"==================",CRLF);
  UB_Uart_SendString(COM6," ",CRLF);

  UB_Systick_Pause_ms(500);

  // Port-A
  UB_Uart_SendString(COM6,"check Port-A[0-3]",CRLF);
  pin_maske=0x0001;
  for(n=0;n<16;n++) {
    if((GPIOA_PINS&pin_maske)!=0) {
      set_gpio_out(GPIOA, pin_maske);
      soll_maske=GPIOA_PINS;
      soll_maske&=~pin_maske;
      c1=check_gpio_hi(GPIOA, soll_maske);
      c2=check_gpio_hi(GPIOE, GPIOE_PINS);
      c3=check_gpio_hi(GPIOF, GPIOF_PINS);
      c4=check_gpio_hi(GPIOG, GPIOG_PINS);
      c5=check_gpio_hi(GPIOH, GPIOH_PINS);
      c6=check_gpio_hi(GPIOI, GPIOI_PINS);
      set_gpio_in(GPIOA, pin_maske);
      if((c1!=0) || (c2!=0) || (c3!=0) || (c4!=0) || (c5!=0) || (c6!=0)) {
        UB_Uart_SendString(COM6,"Port-A error !!",CRLF);
        sprintf(buf,"first error at PA[%d]",n);
        UB_Uart_SendString(COM6,buf,CRLF);
        return(n+1);
      }
    }
    pin_maske<<=1;
  }
  UB_Uart_SendString(COM6,"Port-A ok",CRLF);
  // Port-B
  UB_Uart_SendString(COM6,"skip Port-B",CRLF);
  // Port-C
  UB_Uart_SendString(COM6,"skip Port-C",CRLF);
  // Port-D
  UB_Uart_SendString(COM6,"skip Port-D",CRLF);
  // Port-E
  UB_Uart_SendString(COM6,"check Port-E[4-6]",CRLF);
  pin_maske=0x0001;
  for(n=0;n<16;n++) {
    if((GPIOE_PINS&pin_maske)!=0) {
      set_gpio_out(GPIOE, pin_maske);
      soll_maske=GPIOE_PINS;
      soll_maske&=~pin_maske;
      c1=check_gpio_hi(GPIOE, soll_maske);
      c2=check_gpio_hi(GPIOA, GPIOA_PINS);
      c3=check_gpio_hi(GPIOF, GPIOF_PINS);
      c4=check_gpio_hi(GPIOG, GPIOG_PINS);
      c5=check_gpio_hi(GPIOH, GPIOH_PINS);
      c6=check_gpio_hi(GPIOI, GPIOI_PINS);
      set_gpio_in(GPIOE, pin_maske);
      if((c1!=0) || (c2!=0) || (c3!=0) || (c4!=0) || (c5!=0) || (c6!=0)) {
        UB_Uart_SendString(COM6,"Port-E error !!",CRLF);
        sprintf(buf,"first error at PE[%d]",n);
        UB_Uart_SendString(COM6,buf,CRLF);
        return(n+1);
      }
    }
    pin_maske<<=1;
  }
  UB_Uart_SendString(COM6,"Port-E ok",CRLF);
  // Port-F
  UB_Uart_SendString(COM6,"check Port-F[10]",CRLF);
  pin_maske=0x0001;
  for(n=0;n<16;n++) {
    if((GPIOF_PINS&pin_maske)!=0) {
      set_gpio_out(GPIOF, pin_maske);
      soll_maske=GPIOF_PINS;
      soll_maske&=~pin_maske;
      c1=check_gpio_hi(GPIOF, soll_maske);
      c2=check_gpio_hi(GPIOE, GPIOE_PINS);
      c3=check_gpio_hi(GPIOA, GPIOA_PINS);
      c4=check_gpio_hi(GPIOG, GPIOG_PINS);
      c5=check_gpio_hi(GPIOH, GPIOH_PINS);
      c6=check_gpio_hi(GPIOI, GPIOI_PINS);
      set_gpio_in(GPIOF, pin_maske);
      if((c1!=0) || (c2!=0) || (c3!=0) || (c4!=0) || (c5!=0) || (c6!=0)) {
        UB_Uart_SendString(COM6,"Port-F error !!",CRLF);
        sprintf(buf,"first error at PF[%d]",n);
        UB_Uart_SendString(COM6,buf,CRLF);
        return(n+1);
      }
    }
    pin_maske<<=1;
  }
  UB_Uart_SendString(COM6,"Port-F ok",CRLF);
  // Port-G
  UB_Uart_SendString(COM6,"check Port-G[6-7,10-12]",CRLF);
  pin_maske=0x0001;
  for(n=0;n<16;n++) {
    if((GPIOG_PINS&pin_maske)!=0) {
      set_gpio_out(GPIOG, pin_maske);
      soll_maske=GPIOG_PINS;
      soll_maske&=~pin_maske;
      c1=check_gpio_hi(GPIOG, soll_maske);
      c2=check_gpio_hi(GPIOE, GPIOE_PINS);
      c3=check_gpio_hi(GPIOF, GPIOF_PINS);
      c4=check_gpio_hi(GPIOA, GPIOA_PINS);
      c5=check_gpio_hi(GPIOH, GPIOH_PINS);
      c6=check_gpio_hi(GPIOI, GPIOI_PINS);
      set_gpio_in(GPIOG, pin_maske);
      if((c1!=0) || (c2!=0) || (c3!=0) || (c4!=0) || (c5!=0) || (c6!=0)) {
        UB_Uart_SendString(COM6,"Port-G error !!",CRLF);
        sprintf(buf,"first error at PG[%d]",n);
        UB_Uart_SendString(COM6,buf,CRLF);
        return(n+1);
      }
    }
    pin_maske<<=1;
  }
  UB_Uart_SendString(COM6,"Port-G ok",CRLF);
  // Port-H
  UB_Uart_SendString(COM6,"check Port-H[2-3,8-15]",CRLF);
  pin_maske=0x0001;
  for(n=0;n<16;n++) {
    if((GPIOH_PINS&pin_maske)!=0) {
      set_gpio_out(GPIOH, pin_maske);
      soll_maske=GPIOH_PINS;
      soll_maske&=~pin_maske;
      c1=check_gpio_hi(GPIOH, soll_maske);
      c2=check_gpio_hi(GPIOE, GPIOE_PINS);
      c3=check_gpio_hi(GPIOF, GPIOF_PINS);
      c4=check_gpio_hi(GPIOG, GPIOG_PINS);
      c5=check_gpio_hi(GPIOA, GPIOA_PINS);
      c6=check_gpio_hi(GPIOI, GPIOI_PINS);
      set_gpio_in(GPIOH, pin_maske);
      if((c1!=0) || (c2!=0) || (c3!=0) || (c4!=0) || (c5!=0) || (c6!=0)) {
        UB_Uart_SendString(COM6,"Port-H error !!",CRLF);
        sprintf(buf,"first error at PH[%d]",n);
        UB_Uart_SendString(COM6,buf,CRLF);
        return(n+1);
      }
    }
    pin_maske<<=1;
  }
  UB_Uart_SendString(COM6,"Port-H ok",CRLF);
  // Port-I
  UB_Uart_SendString(COM6,"check Port-I[0-2,4-7,9-10]",CRLF);
  pin_maske=0x0001;
  for(n=0;n<16;n++) {
    if((GPIOI_PINS&pin_maske)!=0) {
      set_gpio_out(GPIOI, pin_maske);
      soll_maske=GPIOI_PINS;
      soll_maske&=~pin_maske;
      c1=check_gpio_hi(GPIOI, soll_maske);
      c2=check_gpio_hi(GPIOE, GPIOE_PINS);
      c3=check_gpio_hi(GPIOF, GPIOF_PINS);
      c4=check_gpio_hi(GPIOG, GPIOG_PINS);
      c5=check_gpio_hi(GPIOH, GPIOH_PINS);
      c6=check_gpio_hi(GPIOA, GPIOA_PINS);
      set_gpio_in(GPIOI, pin_maske);
      if((c1!=0) || (c2!=0) || (c3!=0) || (c4!=0) || (c5!=0) || (c6!=0)) {
        UB_Uart_SendString(COM6,"Port-I error !!",CRLF);
        sprintf(buf,"first error at PI[%d]",n);
        UB_Uart_SendString(COM6,buf,CRLF);
        return(n+1);
      }
    }
    pin_maske<<=1;
  }
  UB_Uart_SendString(COM6,"Port-I ok",CRLF);

  return(0);
}


//----------------------------------------------------------------------
int main(void)
{
  uint8_t check1,check2;

  SystemInit(); // Quarz Einstellungen aktivieren

  // init vom sytsick
  UB_Systick_Init();

  // init der uart
  UB_Uart_Init();

  UB_Systick_Pause_ms(500);

  // powerOn string
  UB_Uart_SendString(COM6,"F429-Board",CRLF);
  UB_Uart_SendString(COM6,"TFT-Pin-Check",CRLF);
  UB_Uart_SendString(COM6," ",CRLF);
  UB_Systick_Pause_ms(500);

  //-------------------------------------------
  // Test-1 : check auf Kurzschluss mit GND
  //-------------------------------------------
  init_ports_pullup();
  UB_Systick_Pause_ms(500);
  check1=check_ports_hi();
  UB_Systick_Pause_ms(500);

  //-------------------------------------------
  // Test-2 : check auf Kurzschluss mit VCC
  //-------------------------------------------  
  init_ports_pulldown();
  UB_Systick_Pause_ms(500);
  check2=check_ports_lo();
  UB_Systick_Pause_ms(500);

  if((check1==0) && (check2==0)) {
    //-------------------------------------------
    // nur wenn Test1+2 keinen Fehler hatte    
    // Test-3 : check auf Kurzschluss von GPIO zu GPIO
    //-------------------------------------------  
    init_ports_pullup();
    UB_Systick_Pause_ms(500);
    check1=check_all_gpios();
    UB_Systick_Pause_ms(500);
    if(check1==0) {
      UB_Uart_SendString(COM6," ",CRLF);
      UB_Uart_SendString(COM6,"pin check passed !!",CRLF);
    }
    else {
      UB_Uart_SendString(COM6," ",CRLF);
      UB_Uart_SendString(COM6,"pin check failed !!",CRLF);
      UB_Uart_SendString(COM6,"please, repair errors",CRLF);
    }
  }
  else {
    UB_Uart_SendString(COM6," ",CRLF);
    UB_Uart_SendString(COM6,"pin check failed !!",CRLF);
    UB_Uart_SendString(COM6,"please, repair errors",CRLF);
  }

  UB_Uart_SendString(COM6," ",CRLF);
  UB_Uart_SendString(COM6,"====================",CRLF);
  UB_Uart_SendString(COM6,"end of TFT-Pin-Check",CRLF);
  UB_Uart_SendString(COM6,"====================",CRLF);

  while(1) {

  }
}

